import java.io.*;
import java.net.DatagramPacket;

public class GameRemoteAccessServer extends GameRemoteAccess
{

    GameRemoteAccessServer(int nPort)
    {
        super(nPort);
        m_cIRemoteGame = null;
    }

    void OnPacketReceived(DatagramPacket cPacket)
    {
        super.OnPacketReceived(cPacket);
        if(m_cIRemoteGame != null)
            m_cIRemoteGame.OnRemoteMessageReceived();
    }

    void Acquire(IRemoteGame cIRemoteGame)
    {
        m_cIRemoteGame = cIRemoteGame;
    }

    void Destroy()
    {
        Acquire(null);
    }

    public static GameRemoteAccessServer GetInstance()
    {
        return m_cSingleton;
    }

    void HandlePacket(DatagramPacket cReceivedPacket)
        throws IOException
    {
        ByteArrayOutputStream cOutputData = new ByteArrayOutputStream();
        DataOutputStream cOutputStream = new DataOutputStream(cOutputData);
        DataInputStream cInputStream = new DataInputStream(new ByteArrayInputStream(cReceivedPacket.getData()));
        int nCommand = cInputStream.readInt();
        if(m_cIRemoteGame == null)
            switch(nCommand)
            {
            case 0: // '\0'
            case 2: // '\002'
            case 4: // '\004'
                cOutputStream.writeInt(6);
                RemoteQueryData cData = new RemoteQueryData();
                cData.Set("Loading map");
                cData.ToOutputStream(cOutputStream);
                break;
            }
        else
            switch(nCommand)
            {
            case 0: // '\0'
            {
                cOutputStream.writeInt(1);
                RemoteQueryDataStatus cData = new RemoteQueryDataStatus();
                m_cIRemoteGame.RemoteQuery_OnWantStatus(cData);
                cData.ToOutputStream(cOutputStream);
                break;
            }

            case 2: // '\002'
            {
                cOutputStream.writeInt(3);
                RemoteQueryDataRules cData = new RemoteQueryDataRules();
                m_cIRemoteGame.RemoteQuery_OnWantRules(cData);
                cData.ToOutputStream(cOutputStream);
                break;
            }

            case 4: // '\004'
            {
                cOutputStream.writeInt(5);
                RemoteQueryDataPlayers cData = new RemoteQueryDataPlayers();
                m_cIRemoteGame.RemoteQuery_OnWantPlayers(cData);
                cData.ToOutputStream(cOutputStream);
                break;
            }
            }
        SendAnswerPacket(cReceivedPacket, cOutputData);
    }

    public static void DeleteInstance()
    {
        if(m_cSingleton != null)
            m_cSingleton.Destroy();
        m_cSingleton = null;
    }

    public static GameRemoteAccessServer CreateInstance(int nPort)
    {
        m_cSingleton = new GameRemoteAccessServer(nPort);
        return m_cSingleton;
    }

    private static GameRemoteAccessServer m_cSingleton = null;
    IRemoteGame m_cIRemoteGame;

}
